# Generated by Django 4.0.4 on 2022-05-20 21:04

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PostDeployAction',
            fields=[
                ('id', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('uuid', models.UUIDField(default=uuid.uuid4)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('started_at', models.DateTimeField(blank=True, null=True)),
                ('completed_at', models.DateTimeField(blank=True, null=True)),
                ('auto', models.BooleanField(default=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('available', models.BooleanField(default=True)),
                ('done', models.BooleanField(default=False)),
                ('task_id', models.UUIDField(blank=True, null=True)),
                ('message', models.TextField(blank=True, null=True)),
            ],
        ),
    ]
